#!/usr/bin/env perl
#!/bin/env perl

## Name: makeGFFprof.pl
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Create scripts for gene reannotation

use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case);

my $help;
my $cluster;
my $bam="instance.bam";
my $path=".";
my $experiment="";
my $pairend;
my $genome="genomesize.txt";
my $node="";
my $src;
my $gff;
my $feature;
my $memory=4;
my $replicate_intron=2;
my $replicate_exon=2;
my $output="output";

my $slot=2;
my $instance;
my $gff_file;
my $line;
my @args;
my @chromosomes;
my @sense = ("NEG", "POS", "UNS");
my @sensesymbol = ("-", "+", ".");
my @sensebio = ("R", "F", "U");

usage() if (($#ARGV<0) or !(GetOptions (
				'help|?' => \$help,
				'cluster' => \$cluster,
				'Pairend' => \$pairend,
				'path=s' => \$path,
				'src=s' => \$src,
				'feature=s' => \$feature,
				'GenomeSize=s' => \$genome,
				'gff=s' => \$gff,
				'memory=i' => \$memory,
				'replicate_intron=i' => \$replicate_intron,
				'replicate_exon=i' => \$replicate_exon,
				'output-prefix=s' => \$output
			    ))
	    or defined($help));

$gff_file = &get_gff_name($gff);

for (my $j=0; $j<3; $j++) {
    open(OUT, ">${output}_$sense[$j].sh");
    if ($cluster) {
	print OUT "#\$ -S /bin/bash\n";
	print OUT "#\$ -N runGFFprof_$sense[$j]\n";
	print OUT "#\$ -V\n";
	print OUT "#\$ -cwd\n";
	print OUT "#\$ -pe parallel 2\n";
	print OUT "#\$ -l h_vmem=".$memory."G\n";
	print OUT "\n";
    }
    else {
	print OUT "#!/usr/bin/env bash\n";
	print OUT "#!/bin/env bash\n";
	print OUT "\n";
    }
    
    print OUT "##variables\n";
    print OUT "SRC=$src\n";
    print OUT "DIR=$path\n";
    print OUT "GFF=$gff_file\n";
    print OUT "FEATURE=$feature\n";
    print OUT "##number of replicates for all libraries is considered as minimum number of reads required for introns to be considered\n";
    print OUT "MINREADINTRON=$replicate_intron\n\n";
    print OUT "##maximum number of replicates for each library is considered as minimum number of reads required for exons to be considered\n";
    print OUT "MINREADEXON=$replicate_exon\n\n";

    print OUT "##change working directory to rnaseq\n";
    print OUT "cd \$DIR/rnaseq\n\n";

    print OUT "##concatenate mappings and sort by read id (long)\n";
    print OUT "samtools cat -h header.sam $sense[$j]_refined_*bam | samtools sort -n -m ${memory}000000000 - $sense[$j]_id_sort\n";
    print OUT "rm $sense[$j]_refined_*bam\n";
    print OUT "echo \"Done: concatenate $sense[$j] mappings and sort by read id\"\n\n";
        
    if ($pairend) {
	print OUT "##join paired-end reads (long)\n";
	print OUT "bamToBed -bed12 -split -i $sense[$j]_id_sort.bam | \$SRC/sortbedid.pl -i STDIN | \$SRC/joinpairbed12.pl -i STDIN --junction $sense[$j]_intron.bed -o $sense[$j]_join.bed\n";
	print OUT "rm $sense[$j]_id_sort.bam\n";
	print OUT "echo \"Done: join paired-end reads $sense[$j]\"\n\n";
    }
    else {
	print OUT "##extract introns (long)\n";
	print OUT "bamToBed -bed12 -split -i $sense[$j]_id_sort.bam | \$SRC/extractintrons.pl --input STDIN --junction $sense[$j]_intron.bed --output $sense[$j]_join.bed\n";
	print OUT "rm $sense[$j]_id_sort.bam\n";
	print OUT "echo \"Done: extract introns\"\n\n";
    }
    ###########################
    print OUT "##join introns (sort long + fusion fast)\n";
    print OUT "sort -k1,1 -k2,2n -k3,3rn $sense[$j]_intron.bed > $sense[$j]_intron_sort.bed\n";
    print OUT "rm $sense[$j]_intron.bed\n";
    print OUT "\$SRC/fusionintron.pl -i $sense[$j]_intron_sort.bed -o $sense[$j]_intron_fusion.bed --mincov \$MINREADINTRON\n";
    print OUT "rm $sense[$j]_intron_sort.bed\n"; 
    print OUT "echo \"Done: join introns $sense[$j]\"\n\n";
    
    print OUT "##sort bed by chromosomes (long)\n";
    print OUT "sort -k1,1 $sense[$j]_join.bed > $sense[$j]_chr_sort_join.bed\n";
    print OUT "rm $sense[$j]_join.bed\n";
    print OUT "echo \"Done: sort bed by chromosomes $sense[$j]\"\n\n";
    
    print OUT "##compute genome coverage (long)\n";
    print OUT "if [ -s \"$sense[$j]_chr_sort_join.bed\" ]\n";
    print OUT "then\n";
    print OUT "genomeCoverageBed -i $sense[$j]_chr_sort_join.bed -bg -g $genome > $sense[$j]_nosplit.tsv\n";
    print OUT "else\n";
    print OUT "echo -n > $sense[$j]_nosplit.tsv\n";
    print OUT "fi\n";
    print OUT "rm $sense[$j]_chr_sort_join.bed\n";
    print OUT "echo \"Done: compute genome coverage $sense[$j]\"\n\n";
    
    print OUT "##find segmentation (fast)\n";
    print OUT "\$SRC/segment.pl -i $sense[$j]_nosplit.tsv --min \$MINREADEXON -o $sense[$j]_segment.tsv\n";
    print OUT "rm $sense[$j]_nosplit.tsv\n";
    print OUT "echo \"Done: find segmentation $sense[$j]\"\n\n";

    print OUT "##get gff file with spliced transcripts (fast)\n";
    print OUT "\$SRC/getsplicedtsc.pl -i $sense[$j]_segment.tsv --junction $sense[$j]_intron_fusion.bed --min \$MINREADINTRON --strand $sensesymbol[$j] --prefix NEW_RNA_$sensebio[$j] -o $sense[$j].\$GFF\n";
    print OUT "rm $sense[$j]_segment.tsv $sense[$j]_intron_fusion.bed\n";
    print OUT "\$SRC/separatetscexon.pl -i $sense[$j].\$GFF -t $sense[$j]_tsc.gff\n";
    print OUT "echo \"Done: get gff file with spliced transcripts $sense[$j]\"\n\n";

    close(OUT);
}

exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: see configure.sh \n";
    exit 1;
}

## get gff filename
sub get_gff_name {
    my $gff = $_[0];
    my $lastslash = rindex($gff, "/");
    
    $gff = substr($gff, $lastslash+1);
    
    return $gff;
}
