#!/usr/bin/env perl
#!/bin/env perl

## Name: makeInit.pl
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Create scripts for bam analysis in each library

use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case);

my $help;
my $cluster;
my $wholeDE;
my $bam="instance.bam";
my $path=".";
my $pairend;
my $genome="genomesize.txt";
my $node="1";
my $src;
my $gff;
my $librarytype="fr-unstranded";
my $insertsize;
my $memory=2;
my $output="output.sh";
my $indexing="";
my $cpu=1;

my $slot=2;
my $instance;

usage() if (($#ARGV<0) or !(GetOptions (
				'help|?' => \$help,
				'cluster' => \$cluster,
				'wholeDE' => \$wholeDE,
				'indexing' => \$indexing,
				'Pairend' => \$pairend,
				'bam=s' => \$bam,
				'path=s' => \$path,
				'node=s' => \$node,
				'src=s' => \$src,
				'gff=s' => \$gff,
				'library-type=s' => \$librarytype,
				'GenomeSize=s' => \$genome,
				'MaxInsertSize=i' => \$insertsize,
				'memory=i' => \$memory,
				'CPU=i' => \$cpu,
				'output=s' => \$output
			    ))
	    or defined($help));

$instance = &get_bam_name($bam);

open(OUT, ">$output");
if ($cluster) {
    print OUT "#\$ -S /bin/bash\n";
    print OUT "#\$ -N runInit".$instance."\n";
    print OUT "#\$ -V\n";
    print OUT "#\$ -cwd\n";
    print OUT "##\$ -q main.q\@node".$node."\n";
    print OUT "#\$ -pe parallel ".$slot."\n";
    print OUT "#\$ -l h_vmem=".$memory."G\n";
    print OUT "\n";
}
else {
    print OUT "#!/usr/bin/env bash\n";
    print OUT "#!/bin/env bash\n";
    print OUT "\n";
}

print OUT "## Note:\n";
print OUT "## - trimread.pl can be used to trim reads with constant trimming sizes at the beginning and at the end of reads.\n";
print OUT "## - use tophat2 or tophat to do mapping\n\n";

print OUT "##change working directory to rnaseq\n";
print OUT "cd $path/rnaseq\n\n";

if ($pairend) {
    print OUT "##get properly paired mapping and add XS tag(average); separate positive, negative and unstranded reads (average)\n";
    print OUT "samtools view -H $bam | head -1 > head1${instance}.sam\n";
    print OUT "sorttype=`grep SO:queryname head1${instance}.sam`\n";
    print OUT "if [ ! \"\$sortype\" ]\n";
    print OUT "then\n";
    print OUT "samtools view -f 0x02 -hbu $bam | samtools sort -no -m ${memory}000000000 - id_sort_".$instance." | samtools view -h - | $src/addXS.pl -i STDIN | $src/separatestrand.pl -i STDIN --neg NEG_".$instance.".sam --pos POS_".$instance.".sam --uns UNS_".$instance.".sam\n";
    print OUT "else\n";
    print OUT "samtools view -f 0x02 -hbu $bam | samtools view -h - | $src/addXS.pl -i STDIN | $src/separatestrand.pl -i - --neg NEG_".$instance.".sam --pos POS_".$instance.".sam --uns UNS_".$instance.".sam\n";
    print OUT "fi\n";
}
else {
    print OUT "##add XS tag (fast)\n";
    print OUT "samtools view -H $bam > XS_".$instance.".sam\n";
    if ($librarytype eq "fr-firststrand") {
	print OUT "samtools view $bam | awk \'{if (\$0 ~ /XS:A:/) print \$0\; else {if (\$2==16) print \$0\"\\tXS:A:+\"\; else print \$0\"\\tXS:A:-\"\;}}\' >> XS_".$instance.".sam\n";
    }
    elsif ($librarytype eq "fr-secondstrand") {
	print OUT "samtools view $bam | awk \'{if (\$0 ~ /XS:A:/) print \$0\; else {if (\$2==16) print \$0\"\\tXS:A:-\"\; else print \$0\"\\tXS:A:+\"\;}}\' >> XS_".$instance.".sam\n";
    }
    else {
	print OUT "samtools view $bam >> XS_".$instance.".sam\n";
    }
    print OUT "$src/separatestrand.pl -i XS_".$instance.".sam --neg NEG_".$instance.".sam --pos POS_".$instance.".sam --uns UNS_".$instance.".sam\n";
    print OUT "rm XS_".$instance.".sam\n";
}
print OUT "echo \"Done: separate positive, negative and unstranded reads\"\n\n";

if ($pairend) {
    print OUT "##remove wrong paired mapping (average)\n";
    print OUT "echo \"samtools view -bS NEG_".$instance.".sam | samtools sort -no -m ${memory}000000000 - NEG_id_sort_".$instance." | samtools view -h - | $src/rmwrongpairmap.pl -i STDIN -m ".$insertsize." -o NEG_refined_".$instance.".sam\" > $path/script/sense_init_neg_".$instance.".sh\n";
    print OUT "echo \"samtools view -bS NEG_refined_".$instance.".sam -o NEG_refined_".$instance.".bam\" >> $path/script/sense_init_neg_".$instance.".sh\n";
    print OUT "echo \"rm NEG_".$instance.".sam\" >> $path/script/sense_init_neg_".$instance.".sh\n";

    print OUT "echo \"samtools view -bS POS_".$instance.".sam | samtools sort -no -m ${memory}000000000 - POS_id_sort_".$instance." | samtools view -h - | $src/rmwrongpairmap.pl -i STDIN -m ".$insertsize." -o POS_refined_".$instance.".sam\" > $path/script/sense_init_pos_".$instance.".sh\n";
    print OUT "echo \"samtools view -bS POS_refined_".$instance.".sam -o POS_refined_".$instance.".bam\" >> $path/script/sense_init_pos_".$instance.".sh\n";
    print OUT "echo \"rm POS_".$instance.".sam\" >> $path/script/sense_init_pos_".$instance.".sh\n";

    print OUT "echo \"samtools view -bS UNS_".$instance.".sam | samtools sort -no -m ${memory}000000000 - UNS_id_sort_".$instance." | samtools view -h - | $src/rmwrongpairmap.pl -i STDIN -m ".$insertsize." -o UNS_refined_".$instance.".sam\" > $path/script/sense_init_uns_".$instance.".sh\n";
    print OUT "echo \"samtools view -bS UNS_refined_".$instance.".sam -o UNS_refined_".$instance.".bam\" >> $path/script/sense_init_uns_".$instance.".sh\n";
    print OUT "echo \"rm UNS_".$instance.".sam\" >> $path/script/sense_init_uns_".$instance.".sh\n";
    
    print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
    print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
    print OUT "echo \"Done: remove wrong paired mapping\"\n\n";
}
else {
    print OUT "##remove wrong paired mapping: single end! (fast)\n";
    print OUT "echo \"samtools view -bS NEG_".$instance.".sam -o NEG_refined_".$instance.".bam\" > $path/script/sense_init_neg_".$instance.".sh\n";
    print OUT "echo \"mv NEG_".$instance.".sam NEG_refined_".$instance.".sam\" >> $path/script/sense_init_neg_".$instance.".sh\n";

    print OUT "echo \"samtools view -bS POS_".$instance.".sam -o POS_refined_".$instance.".bam\" > $path/script/sense_init_pos_".$instance.".sh\n";
    print OUT "echo \"mv POS_".$instance.".sam POS_refined_".$instance.".sam\" >> $path/script/sense_init_pos_".$instance.".sh\n";

    print OUT "echo \"samtools view -bS UNS_".$instance.".sam -o UNS_refined_".$instance.".bam\" > $path/script/sense_init_uns_".$instance.".sh\n";
    print OUT "echo \"mv UNS_".$instance.".sam UNS_refined_".$instance.".sam\" >> $path/script/sense_init_uns_".$instance.".sh\n";

    print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
    print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
    print OUT "echo \"Done: remove wrong paired mapping: single end!\"\n\n";
}

print OUT "##flagstat (fast)\n";
print OUT "echo \"samtools flagstat NEG_refined_".$instance.".bam \> NEG_refined_".$instance.".sta\" > $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"samtools flagstat POS_refined_".$instance.".bam \> POS_refined_".$instance.".sta\" > $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"samtools flagstat UNS_refined_".$instance.".bam \> UNS_refined_".$instance.".sta\" > $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
print OUT "echo \"Done: flagstat\"\n\n";

if ($indexing) {
    print OUT "##create index (long)\n";
    print OUT "samtools cat -h header.sam -o refined_".$instance.".bam NEG_refined_".$instance.".bam POS_refined_".$instance.".bam UNS_refined_".$instance.".bam\n";
    print OUT "if [ -e sort_".$instance.".bam ]\n";
    print OUT "then\n";
    print OUT "rm sort_".$instance.".bam\n";
    print OUT "fi\n";
    print OUT "samtools sort -m ${memory}000000000 refined_".$instance.".bam sort_".$instance."\n";
    print OUT "genomeCoverageBed -bg -ibam sort_".$instance.".bam -g ".$genome." > sort_".$instance.".wig\n";
    print OUT "samtools index sort_".$instance.".bam\n";
    print OUT "echo \"Done: create index\"\n\n";
}

if ($pairend) {
    print OUT "##join paired-end reads (average)\n";
    print OUT "echo \"bamToBed -bed12 -split -i NEG_refined_".$instance.".bam | $src/sortbedid.pl -i STDIN | $src/joinpairbed12.pl -i STDIN -o NEG_join_".$instance.".bed\" > $path/script/sense_init_neg_".$instance.".sh\n";
    print OUT "echo \"bamToBed -bed12 -split -i POS_refined_".$instance.".bam | $src/sortbedid.pl -i STDIN | $src/joinpairbed12.pl -i STDIN -o POS_join_".$instance.".bed\" > $path/script/sense_init_pos_".$instance.".sh\n";
    print OUT "echo \"bamToBed -bed12 -split -i UNS_refined_".$instance.".bam | $src/sortbedid.pl -i STDIN | $src/joinpairbed12.pl -i STDIN -o UNS_join_".$instance.".bed\" > $path/script/sense_init_uns_".$instance.".sh\n";
    print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
    print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
    print OUT "echo \"Done: join paired-end reads\"\n\n";
}
else {
    print OUT "##join paired-end reads: single end! (fast)\n";
    print OUT "echo \"bamToBed -bed12 -split -i NEG_refined_".$instance.".bam \> NEG_join_".$instance.".bed\" > $path/script/sense_init_neg_".$instance.".sh\n";
    print OUT "echo \"bamToBed -bed12 -split -i POS_refined_".$instance.".bam \> POS_join_".$instance.".bed\" > $path/script/sense_init_pos_".$instance.".sh\n";
    print OUT "echo \"bamToBed -bed12 -split -i UNS_refined_".$instance.".bam \> UNS_join_".$instance.".bed\" > $path/script/sense_init_uns_".$instance.".sh\n";
    print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
    print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
    print OUT "echo \"Done: join paired-end reads: single end!\"\n\n";
}

print OUT "##concatenate unstranded to stranded mappings\n";
if ($wholeDE) {
    print OUT "echo \"cat NEG_refined_".$instance.".sam UNS_refined_".$instance.".sam \> UNS_NEG_refined_".$instance.".sam\" > $path/script/sense_init_neg_".$instance.".sh\n";
    print OUT "echo \"cat POS_refined_".$instance.".sam UNS_refined_".$instance.".sam \> UNS_POS_refined_".$instance.".sam\" > $path/script/sense_init_pos_".$instance.".sh\n";
    print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
    print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
}
print OUT "echo \"rm NEG_refined_".$instance.".sam\" > $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"rm POS_refined_".$instance.".sam\" > $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"cat NEG_join_".$instance.".bed UNS_join_".$instance.".bed \> UNS_NEG_join_".$instance.".bed\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"rm NEG_join_".$instance.".bed\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"cat POS_join_".$instance.".bed UNS_join_".$instance.".bed \> UNS_POS_join_".$instance.".bed\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"rm POS_join_".$instance.".bed\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
print OUT "echo \"Done: concatenate unstranded to stranded mappings\"\n\n";

print OUT "##sort bed by chromosomes (long)\n";
print OUT "echo \"sort -k1,1 -k2,2n UNS_NEG_join_".$instance.".bed \> UNS_NEG_sort_".$instance.".bed\" > $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"rm UNS_NEG_join_".$instance.".bed\" >> $path/script/sense_init_neg_".$instance.".sh\n";

print OUT "echo \"sort -k1,1 -k2,2n UNS_POS_join_".$instance.".bed \> UNS_POS_sort_".$instance.".bed\" > $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"rm UNS_POS_join_".$instance.".bed\" >> $path/script/sense_init_pos_".$instance.".sh\n";

print OUT "echo \"sort -k1,1 -k2,2n UNS_join_".$instance.".bed \> UNS_sort_".$instance.".bed\" > $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"rm UNS_join_".$instance.".bed\" >> $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
print OUT "echo \"Done: sort bed by chromosomes\"\n\n";

print OUT "##compute genome coverage (average)\n";
print OUT "echo \"if [ -s \"UNS_NEG_sort_".$instance.".bed\" ]\" > $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"then\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"genomeCoverageBed -i UNS_NEG_sort_".$instance.".bed -g ".$genome." -dz -split \> UNS_NEG_".$instance.".tsv\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"else\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"echo -n \> UNS_NEG_".$instance.".tsv\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"fi\" >> $path/script/sense_init_neg_".$instance.".sh\n";
print OUT "echo \"rm UNS_NEG_sort_".$instance.".bed\" >> $path/script/sense_init_neg_".$instance.".sh\n";

print OUT "echo \"if [ -s \"UNS_POS_sort_".$instance.".bed\" ]\" > $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"then\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"genomeCoverageBed -i UNS_POS_sort_".$instance.".bed -g ".$genome." -dz -split \> UNS_POS_".$instance.".tsv\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"else\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"echo -n \> UNS_POS_".$instance.".tsv\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"fi\" >> $path/script/sense_init_pos_".$instance.".sh\n";
print OUT "echo \"rm UNS_POS_sort_".$instance.".bed\" >> $path/script/sense_init_pos_".$instance.".sh\n";

print OUT "echo \"if [ -s \"UNS_sort_".$instance.".bed\" ]\" > $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"then\" >> $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"genomeCoverageBed -i UNS_sort_".$instance.".bed -g ".$genome." -dz -split \> UNS_".$instance.".tsv\" >> $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"else\" >> $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"echo -n \> UNS_".$instance.".tsv\" >> $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"fi\" >> $path/script/sense_init_uns_".$instance.".sh\n";
print OUT "echo \"rm UNS_sort_".$instance.".bed\" >> $path/script/sense_init_uns_".$instance.".sh\n";

print OUT "ls $path/script/sense_init_\*_${instance}.sh | xargs -i --max-procs=$cpu sh -c \"sh {}\"\n";
print OUT "rm $path/script/sense_init_\*_${instance}.sh\n";
print OUT "echo \"Done: compute genome coverage\"\n\n";

close(OUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: see configure.sh \n";
    exit 1;
}

## remove extension bam or accepted_hits.bam
sub get_bam_name {
    my $bam = $_[0];
    my $extension;
    my $lastslash = rindex($bam, "/");
    
    $bam = substr($bam, $lastslash+1);
    $extension = substr($bam, -4);
    if ($extension eq ".bam") {
	$bam = substr($bam, 0, -4);
	$extension = substr($bam, -13);
	if ($extension eq "accepted_hits") {
	    $bam = substr($bam, 0, -13);
	    $extension = substr($bam, -1);
	    if ($extension eq "." || $extension eq "_") {
		$bam = substr($bam, 0, -1);
	    }
	}
    }
    else {
	print "\tError: bam file is required\n";
	exit 1;
    }
    
    return $bam;
}
