#!/usr/bin/env bash
#!/bin/env bash

## Name: preprof.sh
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Data preparation


ECHO_E="echo -e"
PREFIX="."
PAIREND_OPTION=""
MAX=1

usage() {
    $ECHO_E "Usage: ./preprof.sh [OPTIONS]\n"
    $ECHO_E "Optional:"
    $ECHO_E "-C|--CPU MAX"
    $ECHO_E "\tRun up to MAX processes in parallel. By default, MAX=1."
    $ECHO_E "-h|-?|--help"
    $ECHO_E "\tPrint this help."
    $ECHO_E "-p|--prefix PREFIX"
    $ECHO_E "\tThis is the same directory as PREFIX given in configuration step. All generated scripts are located in PREFIX/script. By default,PREFIX=."
    $ECHO_E "-P|--Pair"
    $ECHO_E "\tThis option is required to analyze paired-end reads. By default, single-end reads are considered.\n"
    
    exit 1
}

if [ ! $1 ]
then
    usage
fi

while :
do
    case $1 in
	-h | --help | -\?)
	    usage
	    ;;
	-p | --prefix)
	    PREFIX=$2
	    shift 2
	    ;;
	--prefix=*)
            PREFIX=${1#*=} #delete everything up till '='
            shift
            ;;
        -P | --Pair)
            PAIREND_OPTION="--Pairend"
            shift
            ;;
	-C | --CPU)
	    MAX=$2
	    shift 2
	    ;;
	--CPU=*)
            MAX=${1#*=}
            shift
            ;;
	-*)
	    $ECHO_ "Unknown option: $1" >&2
	    shift
	    ;;
	--) #end of all options
	    shift
	    break
	    ;;
	*) #no more options, stop while loop
	    break
	    ;;
    esac
done

cd $PREFIX
PREFIX=$PWD
cd - > /dev/null

$ECHO_E "\nPREPROF..."

## run in parallel init_*
find $PREFIX/script -name init_\* | xargs -i --max-procs=$MAX sh -c "sh {}"

## complete description file
$ECHO_E "label\tfile\texperiment\tsize" > $PREFIX/rnaseq/config.tsv
tail -n +2 $PREFIX/rnaseq/description.txt > $PREFIX/rnaseq/tmp_description
while read line
do
    DESCARR=(${line//"\t"/ })
    instance=`basename ${DESCARR[1]}`

    neg=`awk '{if ($4=="mapped") print $1}' $PREFIX/rnaseq/NEG_refined_${instance}.sta`
    pos=`awk '{if ($4=="mapped") print $1}' $PREFIX/rnaseq/POS_refined_${instance}.sta`
    uns=`awk '{if ($4=="mapped") print $1}' $PREFIX/rnaseq/UNS_refined_${instance}.sta`
    libsize=`expr $neg + $pos`
    libsize=`expr $libsize + $uns`

    if [ $PAIREND_OPTION ]
    then
	libsize=`expr $libsize / 2`
    fi

    $ECHO_E "$line\t$libsize" >>  "$PREFIX/rnaseq/config.tsv"
done < $PREFIX/rnaseq/tmp_description
